@@ -32,16 +32,23 @@ func TestHTML(t *testing.T) {
 		{`<html><head></head><body>x</body></html>`, `x`},
 		{`<meta http-equiv="content-type" content="text/html; charset=utf-8">`, `<meta charset=utf-8>`},
 		{`<meta http-equiv="Content-Type" content="text/html; charset=UTF-8" />`, `<meta charset=utf-8>`},
+		{`<meta http-equiv=" content-type " content=" text/html;  charset=utf-8 ">`, `<meta charset=utf-8>`},
 		{`<meta http-equiv="Content-Security-Policy" content="default-src 'self'; img-src https://*; child-src 'none';">`, `<meta http-equiv=content-security-policy content="default-src 'self'; img-src https://*; child-src 'none';">`},
 		{`<meta name="keywords" content="a, b">`, `<meta name=keywords content="a,b">`},
+		//{`<meta name="keywords" content=" a,  b ">`, `<meta name=keywords content="a,b">`}, // TODO
 		{`<meta name="viewport" content="width = 996" />`, `<meta name=viewport content="width=996">`},
 		{`<span attr="test"></span>`, `<span attr=test></span>`},
 		{`<span attr='test&apos;test'></span>`, `<span attr="test'test"></span>`},
 		{`<span attr="test&quot;test"></span>`, `<span attr='test"test'></span>`},
-		{`<span attr='test""&apos;&amp;test'></span>`, `<span attr='test""&#39;&amp;test'></span>`},
+		{`<span attr='test""&apos;&amp;test'></span>`, `<span attr='test""&#39;&test'></span>`},
 		{`<span attr="test/test"></span>`, `<span attr=test/test></span>`},
 		{`<span attr="test/"></span>`, `<span attr=test/></span>`},
-		{`<span>&amp;</span>`, `<span>&amp;</span>`},
+		{`<span>&amp;</span>`, `<span>&</span>`},
+		{`<span>&lt;</span>`, `<span>&lt;</span>`},
+		{`<span>&gt;</span>`, `<span>></span>`},
+		{`<code>&quot;&rsquor;</code>`, `<code>"&#8217;</code>`},
+		{`<small>&#160;</small>`, `<small>&#160;</small>`},
+		{`<span name="&lt;&apos;">a</span>`, `<span name="<'">a</span>`},
 		{`<span clear=none method=GET></span>`, `<span></span>`},
 		{`<span onload="javascript:x;"></span>`, `<span onload=x;></span>`},
 		{`<span selected="selected"></span>`, `<span selected></span>`},
@@ -49,6 +56,7 @@ func TestHTML(t *testing.T) {
 		{`<body id="main"></body>`, `<body id=main>`},
 		{`<link href="data:text/plain, data">`, `<link href=data:,+data>`},
 		{`<svg width="100" height="100"><circle cx="50" cy="50" r="40" stroke="green" stroke-width="4" fill="yellow" /></svg>`, `<svg width="100" height="100"><circle cx="50" cy="50" r="40" stroke="green" stroke-width="4" fill="yellow" /></svg>`},
+		{`<span ></span>`, `<span></span>`},
 		{`</span >`, `</span>`},
 		{`<meta name=viewport content="width=0.1, initial-scale=1.0 , maximum-scale=1000">`, `<meta name=viewport content="width=.1,initial-scale=1,maximum-scale=1e3">`},
 		{`<br/>`, `<br>`},
@@ -131,11 +139,12 @@ func TestHTML(t *testing.T) {
 		{`<script><!--<`, `<script><!--<`},
 
 		// bugs
-		{`<p>text</p><br>text`, `<p>text</p><br>text`},                         // #122
-		{`text <img> text`, `text <img> text`},                                 // #89
-		{`text <progress></progress> text`, `text <progress></progress> text`}, // #89
-		{`<pre> <x> a  b </x> </pre>`, `<pre> <x> a  b </x> </pre>`},           // #82
-		{`<svg id="1"></svg>`, `<svg id="1"></svg>`},                           // #67
+		{`<amp-analytics type=adobeanalytics_nativeConfig>`, `<amp-analytics type=adobeanalytics_nativeConfig>`}, // #270
+		{`<p>text</p><br>text`, `<p>text</p><br>text`},                                                           // #122
+		{`text <img> text`, `text <img> text`},                                                                   // #89
+		{`text <progress></progress> text`, `text <progress></progress> text`},                                   // #89
+		{`<pre> <x> a  b </x> </pre>`, `<pre> <x> a  b </x> </pre>`},                                             // #82
+		{`<svg id="1"></svg>`, `<svg id="1"></svg>`},                                                             // #67
 	}
 
 	m := minify.New()
@@ -158,6 +167,30 @@ func TestHTML(t *testing.T) {
 	}
 }
 
+func TestHTMLCSSJS(t *testing.T) {
+	htmlTests := []struct {
+		html     string
+		expected string
+	}{
+		// bugs
+		{`<div style="font-family: Arial, &#39;sans-serif&#39;; font-size: 22px;">`, `<div style=font-family:Arial,sans-serif;font-size:22px>`}, // #272
+		{`<style amp-boilerplate>body{-webkit-animation:-amp-start 8s steps(1,end) 0s 1 normal both;}</style>`, `<style amp-boilerplate>body{-webkit-animation:-amp-start 8s steps(1,end) 0s 1 normal both;}</style>`},
+	}
+
+	m := minify.New()
+	m.AddFunc("text/html", Minify)
+	m.AddFunc("text/css", css.Minify)
+	m.AddFunc("application/javascript", js.Minify)
+	for _, tt := range htmlTests {
+		t.Run(tt.html, func(t *testing.T) {
+			r := bytes.NewBufferString(tt.html)
+			w := &bytes.Buffer{}
+			err := Minify(m, w, r, nil)
+			test.Minify(t, tt.html, err, w.String(), tt.expected)
+		})
+	}
+}
+
 func TestHTMLKeepEndTags(t *testing.T) {
 	htmlTests := []struct {
 		html     string
@@ -240,6 +273,26 @@ func TestHTMLKeepWhitespace(t *testing.T) {
 	}
 }
 
+func TestHTMLKeepQuotes(t *testing.T) {
+	htmlTests := []struct {
+		html     string
+		expected string
+	}{
+		{`<p attr="test">`, `<p attr="test">`},
+	}
+
+	m := minify.New()
+	htmlMinifier := &Minifier{KeepQuotes: true}
+	for _, tt := range htmlTests {
+		t.Run(tt.html, func(t *testing.T) {
+			r := bytes.NewBufferString(tt.html)
+			w := &bytes.Buffer{}
+			err := htmlMinifier.Minify(m, w, r, nil)
+			test.Minify(t, tt.html, err, w.String(), tt.expected)
+		})
+	}
+}
+
 func TestHTMLURL(t *testing.T) {
 	htmlTests := []struct {
 		url      string
@@ -341,8 +394,8 @@ func TestMinifyErrors(t *testing.T) {
 		err  error
 	}{
 		{`<style>abc</style>`, test.ErrPlain},
-		{`<path style="abc"/>`, test.ErrPlain},
-		{`<path onclick="abc"/>`, test.ErrPlain},
+		{`<p style="abc"/>`, test.ErrPlain},
+		{`<p onclick="abc"/>`, test.ErrPlain},
 		{`<svg></svg>`, test.ErrPlain},
 		{`<math></math>`, test.ErrPlain},
 	}
