@@ -2,14 +2,13 @@ package svg
 
 import (
 	"github.com/tdewolff/parse/v2"
-	"github.com/tdewolff/parse/v2/svg"
 	"github.com/tdewolff/parse/v2/xml"
 )
 
 // Token is a single token unit with an attribute value (if given) and hash of the data.
 type Token struct {
 	xml.TokenType
-	Hash    svg.Hash
+	Hash    Hash
 	Data    []byte
 	Text    []byte
 	AttrVal []byte
@@ -39,12 +38,14 @@ func (z *TokenBuffer) read(t *Token) {
 	if t.TokenType == xml.AttributeToken {
 		t.AttrVal = z.l.AttrVal()
 		if len(t.AttrVal) > 1 && (t.AttrVal[0] == '"' || t.AttrVal[0] == '\'') {
-			t.AttrVal = parse.ReplaceMultipleWhitespace(parse.TrimWhitespace(t.AttrVal[1 : len(t.AttrVal)-1])) // quotes will be readded in attribute loop if necessary
+			t.AttrVal = t.AttrVal[1 : len(t.AttrVal)-1] // quotes will be readded in attribute loop if necessary
+			t.AttrVal = parse.ReplaceMultipleWhitespaceAndEntities(t.AttrVal, xml.EntitiesMap, nil)
+			t.AttrVal = parse.TrimWhitespace(t.AttrVal)
 		}
-		t.Hash = svg.ToHash(t.Text)
+		t.Hash = ToHash(t.Text)
 	} else if t.TokenType == xml.StartTagToken || t.TokenType == xml.EndTagToken {
 		t.AttrVal = nil
-		t.Hash = svg.ToHash(t.Text)
+		t.Hash = ToHash(t.Text)
 	} else {
 		t.AttrVal = nil
 		t.Hash = 0
@@ -100,7 +101,7 @@ func (z *TokenBuffer) Shift() *Token {
 
 // Attributes extracts the gives attribute hashes from a tag.
 // It returns in the same order pointers to the requested token data or nil.
-func (z *TokenBuffer) Attributes(hashes ...svg.Hash) []*Token {
+func (z *TokenBuffer) Attributes(hashes ...Hash) []*Token {
 	n := 0
 	for {
 		if t := z.Peek(n); t.TokenType != xml.AttributeToken {
