@@ -69,10 +69,10 @@ func (o *Minifier) Minify(m *minify.M, w io.Writer, r io.Reader, _ map[string]st
 				omitSpace = true
 			}
 		case xml.TextToken:
-			t.Data = parse.ReplaceMultipleWhitespace(t.Data)
+			t.Data = parse.ReplaceMultipleWhitespaceAndEntities(t.Data, EntitiesMap, nil)
 
 			// whitespace removal; trim left
-			if omitSpace && (t.Data[0] == ' ' || t.Data[0] == '\n') {
+			if omitSpace && parse.IsWhitespace(t.Data[0]) {
 				t.Data = t.Data[1:]
 			}
 
@@ -80,7 +80,7 @@ func (o *Minifier) Minify(m *minify.M, w io.Writer, r io.Reader, _ map[string]st
 			omitSpace = false
 			if len(t.Data) == 0 {
 				omitSpace = true
-			} else if t.Data[len(t.Data)-1] == ' ' || t.Data[len(t.Data)-1] == '\n' {
+			} else if parse.IsWhitespace(t.Data[len(t.Data)-1]) {
 				omitSpace = true
 				i := 0
 				for {
@@ -145,8 +145,9 @@ func (o *Minifier) Minify(m *minify.M, w io.Writer, r io.Reader, _ map[string]st
 					return err
 				}
 			} else {
-				// prefer single or double quotes depending on what occurs more often in value
-				val := xml.EscapeAttrVal(&attrByteBuffer, t.AttrVal[1:len(t.AttrVal)-1])
+				val := t.AttrVal[1 : len(t.AttrVal)-1]
+				val = parse.ReplaceEntities(val, EntitiesMap, nil)
+				val = xml.EscapeAttrVal(&attrByteBuffer, val) // prefer single or double quotes depending on what occurs more often in value
 				if _, err := w.Write(val); err != nil {
 					return err
 				}
@@ -168,16 +169,16 @@ func (o *Minifier) Minify(m *minify.M, w io.Writer, r io.Reader, _ map[string]st
 					return err
 				}
 			} else {
-				if _, err := w.Write(t.Text); err != nil {
+				if _, err := w.Write(t.Data); err != nil {
 					return err
 				}
 			}
 		case xml.StartTagCloseVoidToken:
-			if _, err := w.Write(t.Text); err != nil {
+			if _, err := w.Write(t.Data); err != nil {
 				return err
 			}
 		case xml.StartTagClosePIToken:
-			if _, err := w.Write(t.Text); err != nil {
+			if _, err := w.Write(t.Data); err != nil {
 				return err
 			}
 		case xml.EndTagToken:
